<?php

declare(strict_types=1);

namespace Rowlinson\Api;

use GuzzleHttp\Exception\BadResponseException;
use Rowlinson\Api\Exceptions\ApiException;
use Rowlinson\Api\Requests\Basket\AddItemRequest;
use Rowlinson\Api\Requests\Basket\UpdateItemRequest;
use Rowlinson\Api\Requests\Basket\ValidateRequest;
use Rowlinson\Api\Responses\Basket\BasketResponse;
use Rowlinson\Api\Responses\Basket\DeleteItemResponse;
use Rowlinson\Api\Responses\Basket\UpdateItemResponse;
use Rowlinson\Api\Responses\Basket\ValidateResponse;
use Symfony\Component\HttpFoundation\Response;

class BasketClient extends AbstractClient
{
    /**
     * Add a product variant to the currently logged in user's basket
     */
    public function addToBasket(string $sku, int $quantity): void
    {
        $this->post('basket/add_item', new AddItemRequest($sku, $quantity));
    }

    /**
     * Get the currently logged in user's basket
     */
    public function getBasket(): BasketResponse
    {
        $response = $this->get('basket');

        return $this->serializer->deserialize(
            (string)$response->getBody(),
            BasketResponse::class,
            'json'
        );
    }

    /**
     * Updates a single basket item
     */
    public function updateBasketItem(string $variantSku, int $quantity): UpdateItemResponse
    {
        $response = $this->put('basket/update_item', new UpdateItemRequest($variantSku, $quantity));

        return $this->serializer->deserialize(
            (string)$response->getBody(),
            UpdateItemResponse::class,
            'json'
        );
    }

    /**
     * Removes a single item from the users basket
     */
    public function removeBasketItem(string $sku): DeleteItemResponse
    {
        $response = $this->delete("basket/remove_item/$sku");

        return $this->serializer->deserialize(
            (string)$response->getBody(),
            DeleteItemResponse::class,
            'json'
        );
    }

    /**
     * Validate a hypothetical basket of items against an optional organisation (admin only)
     *
     * @param array<array<string,string|int>> $items An array of arrays, [ 'sku' => 'ABC/123', 'quantity' => 12 ]
     */
    public function validate(array $items, ?string $organisation = null): ValidateResponse
    {
        $response = null;

        try {
            $response = $this->post('basket/validate', new ValidateRequest($items, $organisation));
        } catch (ApiException $e) {
            if ($e->getCode() !== Response::HTTP_BAD_REQUEST || !($e->getPrevious() instanceof BadResponseException)) {
                throw $e;
            }

            $response = $e->getPrevious()->getResponse();
        }

        return $this->serializer->deserialize(
            (string)$response->getBody(),
            ValidateResponse::class,
            'json'
        );
    }
}
